#!/usr/bin/env perl
use SynBlast;
our $PROGN=[split(/\//, $0)]->[-1];
$VERSION = "SynBlast-" . $VERSION . "\n${PROGN}'s Time-stamp: <2008-08-19 14:58:09 joe>\nsee ${url} for updates.\n";
#######################################################
#doSyntenyAlignment.pl  - part of the SynBlast package for Assisting the Analysis of Conserved Synteny Information
#Copyright (C) 2008  Joerg Lehmann
# 2006-03-04
# joe@bioinf.uni-leipzig.de
#######################################################

=head1 NAME

doSyntenyAlignment.pl - a tool to visualize and align/evaluate blast (tblastn) hits to the order of query proteins.
Query proteins' annotation must be provided as an annotation file in tab-separated format (I<*.protein.info file>, see bottom).
One or more blast hit tabular output files containing only blast hits within a restricted target location and range are required (should represent putative regions of conserved synteny/gene order).

doSyntenyAlignment.pl is part of the L<SynBlast> package for Assisting the Analysis of Conserved Synteny Information.

=head1 SYNOPSIS

B<doSyntenyAlignment.pl> 
B<-i|infoFile=I<Protein.InfoFile>>

[B<-sortByRatio=I<0|1|2|3>>]
[B<-h|htmlFile=I<ResultHtmlFile>>]
[B<-txtFile=I<ResultTxtFile>>]
[B<-n|number=I<NumberOfResultsInHTML>>]
[B<-r|refFields=I<comma-separated-ploci-fields>>]
[B<-p|plociFields=I<comma-separated-ploci-fields>>]
[B<-q|qlociFields=I<comma-separated-qloci-fields>>]
[B<-e|compara[=I<EnsEMBL_DB_init file>]>]
[B<-comparaAll>]
[B<-globalRank>]
[B<-printColors>]
[B<-showAllRegions>]
[B<-s|mean>]

[B<-blastDir=I<tblastnSourceDir>>]
[B<-blastFilePraefix=I<AlternativeBlastFilePraefix>>]
[B<-wublast>]

[B<-c|sCutOff=I<CutoffBitscore>>]
[B<-o|protOverlap=I<MaxProteinOverlap>>]
[B<-l|lociSizeFactor= I<LociSizeFactor>>]
[B<-mLociSize=I<MaxGlobLociSize>>]
[B<-g|gapCosts=I<gapCost(s)>>]
[B<-m|mismatchCost=I<MismatchCost>>]
[B<-t|misOriPenalty=I<PenaltyFactorMisorientation>>]

[B<-noRelScore>]
[B<-maxQlociDist=I<distance>>]

[B<-ensemblHost=I<EnsEMBL_DB_host>>]
[B<-ensemblUser=I<EnsEMBL_DB_user>>]

[B<-v|verbose>]
[B<--help>] [B<--manual>] [B<--version>]

[[B<-deepChaining>]]
[[B<-snumber=I<NumberOfResultsPerSpeciesInHTML>>]]
[[B<-trackerInputFile=I<TrackerInputFile>>]]
[[B<-trackerN=I<NumberOfExonPerSpecies>>]]
[[B<-trackerFlankS=I<NumberOfNucleotidesToAdd>>]]


SyntenyBlasthitFile(s)

=head1 REQUIRES

L<MySyntenyAln>
L<MySyntenyUtils>,
L<MyUtils>,
L<Getopt::Long>,
L<Pod::Usage>

=head1 DESCRIPTION

The evaluation of synteny regions is done in the following way:

For each protein query found/contained within a synteny region,
the corresponding single blast hits (HSPs) within a limited maximal range (based on the queries genomic range) are stitched together 
covering the protein query more or less (so-called target-loci), and are finally represented by overall sums of Blast bitscores.

A gene order alignment (query loci and target loci positions) is performed for each input synteny region, 
output in HTML or text format contains alignment results and query-target assignments of assembled blast-hit (target) loci 
in rank order according to overall similarity score (alignment score, which is reduced by gap costs), 
or alternatively rank order according to (log)intra-inter-score-ratio sum (see '-sortByRatio' option)

Access to Ensembl databases to display additional information for each assignment can be used (see -compara, -comparaAll options), for comparison.
Determination of the globally-best assembled blast-hit scores (-globalRank option) can be done in order to display the global rank for each assignment.
Note that the original tblastn result files should be available (at least for the reference species; 
see options -blastDir and -blastFilePraefix as well as -globalRank and -noRelScore)

Further options described below.


=head1 ARGUMENTS

=over 5

=item B<-i|infoFile=> I<protein.info-file> [String]

specifies the annotation file containing sequence features (proteins) to be used as reference loci during alignment (reference gene order).
For Ensembl genomes, L<getEnsemblProteins.pl> performs the retrieval automatically.
The format is a tab-separated list of features describing each protein-coding gene of the set, one per line. 
Definition of columns (column names separated by comma) and some more annotation descriptors have to be specified in the head, see following example:

  # protein.info file manually generated
  #!organism=Branchiostoma
  #!coordsystem=scaffold
  #!seqregion=scaffold_89
  #!from=1
  #!to=650000
  #!strand=1
  #!focalgene=89000016
  #!columns=startPos,endPos,geneName,protID,ori,geneID,numExons,protLength
  42268   52505   gene1       fgenesh2_pm.scaffold_89000001   -1      89000001        3       592
  136667  137756  gene2       fgenesh2_pg.scaffold_89000006   1       89000006        2       121
  139176  141632  -       fgenesh2_pg.scaffold_89000007   -1      89000007        5       452
  147110  150418  -       fgenesh2_pg.scaffold_89000009   -1      89000009        1       1103
  #testcomment
  163582  175636  -       fgenesh2_pg.scaffold_89000011   1       89000011        15      649
  ...

=item B<SyntenyBlasthitFile(s)>

The (syntenic) blast hit files to process. Each file is later displayed as an alignment to the reference proteins.
These files should be automatically generated using L<doSyntenyFilter.pl> out of raw blast results in tabular format (either NCBI or WUBLAST format).
The syntenic blast hit files are expected in NCBI-like tabular format.

=head1 OPTIONS

=over 5

=item B<-sortByRatio[=I<0|1|2|3>]>

 To specify the use of ratio sum scores (sum of ratios of intra-/inter-scores), 
 instead of the gene order alignment score, as sorting/ranking criterium for evaluated synteny regions.
 Note that only finally assigned (i.e. assigned in the optimal gene order alignment selected) target loci 
 are taken into account when calculating the alternative scores of a synteny region.
 This can help to better distinguish between orthologs and paralogs.

        0=off,default (sort by gene order alignment score)
	1=ratioSum       (sum of ratios)
	2=logRatioSum    (sum of logarithmized ratios)
	3=focalRatioSum  (sum of ratios weighted with the distance to the focal gene position)

=item B<-h|htmlFile=> I<ResultHtmlFile> [String]

 To output the ranked gene order alignments as tables in html format to the specified file 
 (this also creates a subdirectory for dotplot and alignment graphics as well as assembled blast hits)

=item B<-txtFile=> I<ResultTxtFile> [String]

 To output the ranked gene order alignments as tables in txt format (tab-delimited) to the specified file

=item B<-n|number=> I<NumberOfResultsInHTML> [Integer]

 To limit the number of displayed results in html file.
 This can be used e.g. to only look at the results of  the 10 best evaluated target regions.
 default = 0
 (no limit, i.e. each candidate region is displayed as result, as long as there was any assignment of a target locus possible)


=item B<-r|refFields=> I<comma-separated-ploci-fields> [String]

 To select custom p-loci-data columns displayed/used for the reference overview table in the HTML file
 Note: Column names have to correspond to those used within the annotation file.
	
 default is: "geneName,protID,transID,geneID,protName,chr,startPos,endPos,meanPos,ori,numExons,protLength"
 
 standard/possible field names are (column names created by L<getEnsemblProteins.pl>): 
	chr
	startPos
	endPos
	meanPos (added automatically, if mean flag set)
	protID
	transID
	geneID
	ori
	protLength
	protName
	numExons
	geneName

=item B<-p|plociFields=> I<comma-separated-ploci-fields> [String]

 To select custom ploci-data columns displayed/used for the detailed alignment table in the HTML file
 default is: "protID,geneName,optQloci";
 possible field names are:
	the same names as for the B<-r|refFields=> option +
	optQloci

=item B<-q|qlociFields=> I<comma-separated-qloci-fields> [String]

 To select custom qloci-data columns displayed/used for the detailed alignment table in the HTML file
 default is: "score,chr,startPos,endPos,ori,hits,queryCov,meanIdentity,hitsLength,intra_inter";
 possible field names are:
	score
	ori
	startPos
	endPos
	meanPos (is displayed automatically, if mean flag set)
	chr
	queryCov
	meanIdentity
	hitsCount
	hitsLength
	hits
	compara (is displayed automatically, if compara flag set)
	globalRank (is displayed automatically, if globalRank flag set)
	intra_inter (is displayed automatically, if sortByRatio flag set)

=item B<-e|compara[=I<EnsEMBL_DB_RegistryInitFile> [String]]>

 Flag to enable retrieval and display of Compara results for assigned focal gene loci, 
 i.e. if the target assignment for the focal reference gene (only shown in result overview table) is according to the orthology prediction of the Ensembl Compara database 
 (see 'compara' field in tables).
 Additionally, sequences of Compara-annotated proteins (comparaHits) as well as yet-unannotated (w.r.t. Compara) proteins within the assigned target interval are saved as 
 fasta files into the subdirectory I<<geneID>-homologs>
 Optionally, one can here specify a user-configured Ensembl_DB_RegistryInitFile for use of local Ensembl databases (mysql).
 See http://www.ensembl.org/info/using/api/registry/index.html about the format of such a configuration file.

=item B<-comparaAll>

 Flag to enable display of Compara comparison/result for each assignment in the detailed alignment table. Enables also the B<-compara> flag, if not used explicitely.

=item B<-globalRank[=I<NumberOfBestHitsToReport> [Integer]]>

 Flag to enable the retrieval of globally best hits and display of rank number for each assignment
 (see column 'globalRank' in the detailed alignment table)

=item B<-printColors>

 Flag to generate dotplots graphics with a higher resolution, and additionally as a black-white version

=item B<-showAllRegions>

 Flag to display results for all input regions (even if no alignment/target locus is available); 
 By default, regions with no alignment score are not displayed.



=item B<-blastDir=> I<tblastnSourceDir> [String]

 To specify the directory containing the appropriate tblastn result files,
 needed for the retrieval of the global-best-hit ranks (see option L<-globalRank>).
 The format of file names for the tblastn result files has to be either "QueryID.<SpeciesName>.blastresult" (by default)
 or "<BlastFilePraefix>.<SpeciesName>.blastresult" (if <BlastFilePraefix> is set).
 default = "tblastn" 

=item B<-blastFilePraefix=> I<BlastFilePraefix> [String]

 To specify an alternative blast file praefix and not to look for query specific blast result files,
 but a file named "<BlastFilePraefix>.<SpeciesName>.blastresult"
 Needed for the retrieval of the global-best-hit ranks (see option L<-globalRank>).
 default = undef 
 (There is one query specific blast result file for every single query sequence, named "QueryID.<SpeciesName>.blastresult"

=item B<-wublast>

 Flag to use raw blastresults in WU-BLAST tabular format (instead of NCBI format)
 (only relevant for the raw blast results needed for the globalRank option and the selfscore calculation! Filtered blastresults are always in NCBI-like tabular format!)



=item B<-c|sCutOff=> I<CutoffBitscore> [Integer]

 To set the minimum bitscore that has to be reached by any assembled blast hit to be considered as target locus.
 default = 50  

=item B<-o|protOverlap=> I<MaxProteinOverlap> [Integer]

 To set the maximal amount of positions that neighbored tblastn hits may overlap on the query site to still be considered as 'consistent'.
 default = 30 
 (amino acid positions) 

=item B<-l|lociSizeFactor=> I<LociSizeFactor> [Float]

 To specify the maximal length allowed for a target locus, relative to the query's range.
 default = 2.0 
 (i.e. twice of the query's range/size is used to restrict the length of a target locus;
  3 or 4 may be a better factor not to miss some more distant exon hits for an assembled hit in some cases)

=item B<-mLociSize=> I<MaxGlobLociSize> [Integer]

 To specify a maximally allowed size (genomic extension) for valid target loci 
 default = 0 (not set)
 if set, the maximal size is restricted, but may be even smaller depending on parameter I<-l> (LociSizeFactor)

=item B<-g|gapCosts=> I<gapCost> [String]

 To specify the gap costs for the gene order alignment, 
 (better to speak of gap scores, as values should be negative here!)
 default = "-2e1:-2e1" (gapP[:gapQ])
 Note: It is possible to specify gap costs for insertions in the target sequence of loci (gapP, i.e. gaps in the reference sequence of loci),
       separately from the gap costs for gaps within the sequence of target loci (gapQ).

=item B<-m|mismatchCost=> I<MismatchCost> [String]

 To specify the score (cost) for a mismatch within gene loci alignment.
 Note: This value should be relatively/highly negative in relation to usually occuring match scores and gap scores, 
 to simply not allow mismatches in optimal alignments!
 default = -1000

=item B<-t|misOriPenalty=> I<PenaltyFactorMisorientation> [Integer]

 To specify the fraction of the matching score that is used in case of misorientated loci matching within the alignment; 
 default = 90 (in percent)

=item B<-maxQlociDist=> I<distance> [String]

 To set the maximal distance between neighbored target loci that are not marked orange/black in the dotplot graphics. 
 (This is just to identify biologically more meaningful blocks of contigous target hits more easily, in case of large-sized synteny regions.)
 default = 500000 nt
 (use 0 for the maximal distance between neighbored reference loci multiplied with the <LociSizeFactor>)

=item B<-rlociSizeFactor=> I<RefLociSizeFactor> [Float]>

 The LociSizeFactor for the reference-score retrieval (needed for selfscore, intra/inter-scores calculations) can be specified, separately.
 default = 2.0
 
 Note: The other parameters used to obtain the best and second best within-reference scores (as needed for the blast-score normalization 
 as well as the intra/inter-score calculations) are fixed: sCutoff=0; protOverlap=40; mLociSize=0.

=item B<-noRelScore>

 Flag to disable the use of relative scores for alignment calculations
 default is: the calculated relative score will be used for alignment calculations 
 Note: This is only possible if within-reference blast hit data is available for all reference protein queries. 
       Otherwise those reference entries will have to be removed from the reference set, manually.


=head2 rarely used options:

=item B<-ensemblHost=> I<EnsEMBL_DB_Hostname> [String]

=item B<-ensemblUser=> I<EnsEMBL_DB_Username> [String]

=item B<-s|mean>

 Flag: using the mean CDS(annotation)/HSP(blasthit) position as sort index
 (default is: using the start and end positions and combine overlapping loci)

=item B<-v|verbose>

 verboseFlag: prints intermediate results verbosely (not recommended)

=item B<-snumber=> I<NumberOfResultsPerSpeciesInHTML> [Integer]

 To limit the number of displayed results for each species in html file. [experimental]
 default = 0
 (no limit)

=item B<-trackerInputFile=> I<TrackerInputFile> [String]

 generates inputfile for use with the Tracker program [experimental]

=item B<-trackerN=> I<NumberOfExonPerSpecies> [Integer]

 number of genes per species to write to tracker input file, 
 default = 0 (no tracker input is generated at all)

=item B<-deepChaining> [experimental]

 Flag to use the recursive version considering all possible combinations of HSPs maximizing the assembled hit 
 w.r.t. query overlap size, range factor, and score cutoff parameters (not recommended)
 default is: using an alignment based approach relying on consistent overlappings of putative exon matches! 
 (faster but might miss some HSPs due to overlap with insignificant HSPs) 



=head1 AUTHOR

Joerg Lehmann, <joe@bioinf.uni-leipzig.de>


=head1 AVAILABILITY

http://www.bioinf.uni-leipzig.de/Software/SynBlast/


=head1 LICENSE

This  program is free  software; you can redistribute  it and/or
modify it  under the terms of the GNU  General Public License as
published by the  Free Software Foundation; either version 2, or
(at your option) any later version.

This program is  distributed in the hope that it will be useful,
but WITHOUT  ANY WARRANTY; without even the  implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
General Public License for more details.

You can view the  GNU General Public License, online, at the GNU
Project's homepage; see <http://www.gnu.org/licenses/gpl.html>.

See also L<perlgpl> for more information.

=head1  SEE ALSO

L<SynBlast::MySyntenyAln>,
L<SynBlast::MySyntenyUtils>, 
L<SynBlast::MyUtils>,
L<getEnsemblProteins.pl>,
L<doBlastJobs.pl>,
L<doSyntenyFilter.pl>,
L<getBestBlastHit.pl>

=cut

use strict;
use warnings;

use SynBlast::MySyntenyAln;
use SynBlast::MyUtils;
use Getopt::Long;
use Pod::Usage;
use SynBlast::EnsEMBLaccess;

#use DataBrowser;

$|=1;

my $cmdlineWas = ${main::PROGN} . " " . join(" ", @ARGV);
#$Getopt::Std::STANDARD_HELP_VERSION = 1;

my($infoFile, $htmlFile, $txtFile, $trackFile);
my $htmlLimit=0;
my $htmlSpLimit=0;
my $trackerLimit=0;
my $trackOvl=100;



### some configuration for retrieving globalBestHit (RankData)
my $ablastf=undef;
#$ablastf=$opts{"a"} if(exists($opts{"a"}));
my $bsourceDir="tblastn";
#$sourceDir=$opts{"s"} if(exists($opts{"s"}));
#MyUtils::checkStrEnd(\$sourceDir, "/") && print STDERR "blastresult directory name changed to " . $sourceDir . "\n";



#default values for parameters:
my($scutoff, $protOvl, $globLocSize) = ( 50, 30, 0 );
my($locSizeFac, $rlocSizeFac) = (2.0,  2.0);
my($gapPstr, $mismP, $pMisOri) = ( "-2e1:-2e1", -1e3, 90 );
#output column defaults:
#qfields default is: "score,chr,startPos,endPos,ori,hits,queryCov,meanIdentity,hitsLength,intra_inter";
my($ppfields
   , $pfields
   , $qfields) = ( 
		   join(",", qw(geneName protID transID geneID protName chr startPos endPos meanPos ori numExons protLength)), 
		   join(",", qw(protID geneName optQloci)),
		   join(",", qw(score chr startPos endPos ori hits queryCov meanIdentity hitsLength intra_inter))
		   );

#flags are off per default
my($verboseFl, $bFl, $sFl, $comparaFl, $comparaAllFl, $globalRankFl, $noRelScFl, $ratioFl) = ( 0, 0, 0, undef, undef, undef, 0, undef);
my $deepFl = 0;
my $printcFl=0; ## to use printer-friendly colors in dotplot and alignment graphics
my($ehost, $euser) = ("ensembldb.ensembl.org", "anonymous");
my($help, $version, $man) = (0, 0, 0);
my ($maxDist, $maxDistStr) = (500000, undef);
my $wublastFl=0;
my $showallFl=0;


GetOptions( 'help|?' => \$help,	    'version' => \$version,	    'manual' => \$man,
	    'infoFile|i=s' => \$infoFile,
	    'htmlFile|h=s' => \$htmlFile,
	    'txtFile=s' => \$txtFile,
	    'trackerInputFile=s' => \$trackFile,
	    'trackerFlankS=i' => \$trackOvl,

	    'number|n=i' => \$htmlLimit,
	    'snumber=i' => \$htmlSpLimit,
	    'trackerNumber=i' => \$trackerLimit,

	    'blastDir=s' => \$bsourceDir,
	    'blastFilePraefix=s' => \$ablastf,
	
	
	    'sCutOff|c=i' => \$scutoff,
	    'protOverlap|o=i' => \$protOvl,
	    'lociSizeFactor|l=f' => \$locSizeFac,
	    'rlociSizeFactor=f' => \$rlocSizeFac,
	    'mLociSize=i' => \$globLocSize,
	    'gapCosts|g=s' => \$gapPstr,
	    'mismatchCost|m=i' => \$mismP,
	    'misOriPenalty|t=i' => \$pMisOri, ## misorientation penalty factor
	    ##(fraction in percent that is kept of match score in case of wrong orientation wrt reference locus)
	    'maxQlociDist=s' => \$maxDistStr,
	    
	    'refFields|r=s' => \$ppfields,
    	    'plociFields|p=s' => \$pfields,
    	    'qlociFields|q=s' => \$qfields,
	   
	    'verbose|v' => \$verboseFl,
	#    'selfcomp|b' => \$bFl,
	    'mean|s' => \$sFl,
	    'compara|e:s' => \$comparaFl,    # optional: string of initFile to be used for compara-access!!!
	    'comparaAll' => \$comparaAllFl,
			'globalRank:i' => \$globalRankFl,  # optional: integer for nglobal Parameter (levels of global hits to determine)
			'sortByRatio:i' => \$ratioFl,   ## optional: integer 1=sum of ratios (default), 2=log sum of ratios (like product), 3=weighted sum of ratios (distance to focal gene)
			'noRelScore' => \$noRelScFl,
			'deepChaining' => \$deepFl,
				    
	    'ensemblHost=s' => \$ehost,
	    'ensemblUser=s' => \$euser,
	    'printColors' => \$printcFl,
	    'wublast'=> \$wublastFl,
	    'showAllRegions' => \$showallFl
	    
#	    'initFile=s' => \$initFile
	    ) 
    or pod2usage(-verbose => 0);
VERSION_MESSAGE() if($version || $help);
HELP_MESSAGE() if $help;
exit(1) if($version);
pod2usage(-verbose => 2, -noperldoc => 1) if $man;
pod2usage(-verbose => 0, -message => "$PROGN: Error - no BlastHitFile(s) specified.\n") unless @ARGV>0;
pod2usage(-verbose => 0, -message => "$PROGN: Error - no ReferenceInfoFile specified (-i option).\n") unless defined($infoFile);

$noRelScFl=1 if($bFl);

#### determine release archive string that the target regions should be tagged with (for valid comparisons with Compara DB of API-release=DB-release version) 
my $relStringShouldBe=lc(SynBlast::EnsEMBLaccess::getRelMonthString());


if(! defined($trackFile)) {
  if(defined($htmlFile)) {
    $trackFile=substr($htmlFile, 0, -4) . "tracker";
  }
}

# for compara comparison
my $initF = undef;
$initF = $comparaFl if(defined($comparaFl) && ($comparaFl ne ""));
$comparaFl=1 if(defined($comparaFl));
$comparaFl=1 if(defined($comparaAllFl));
$comparaAllFl=undef if(! $comparaFl);

my $nglobal = 10;
$nglobal = $globalRankFl if(defined($globalRankFl) && (int($globalRankFl)));
$globalRankFl=1 if(defined($globalRankFl));
#$globalRankFl=1 if(defined($ratioFl));

if(defined($ratioFl))  {
		$ratioFl = 1 if($ratioFl eq "");
} else {
	$ratioFl = 0;
}

$qfields .= ",compara" if( $comparaAllFl && (index($qfields, "compara") == -1));
$qfields .= ",globalRank" if( $globalRankFl && (index($qfields, "globalRank") == -1));

#$qfields .= ",intra_inter" if( $globalRankFl && $ratioFl && (index($qfields, "intra_inter") == -1));
$qfields .= ",intra_inter" if($ratioFl && (index($qfields, "intra_inter") == -1));



my $displRatio;
if(index($qfields, "intra_inter") != -1) {
#	$globalRankFl=1;   ### globalRank flag needed for intra_inter ratio calculation
	 $displRatio=1;
 }


$qfields = "meanPos,${qfields}" if( $sFl && (index($qfields, "meanPos") == -1));


my @ppfields= split(/,/, $ppfields);
my @pfields=  split(/,/, $pfields);
my @qfields=  split(/,/, $qfields);
$mismP = int($mismP);
my($gapPp, $gapPq) = split(/:/, $gapPstr);
$gapPq = $gapPp if(! defined($gapPq));
($gapPp, $gapPq) = (int($gapPp), int($gapPq));
#gapPp=gapCost for gap inserts into informant/reference sequence (p-loci)  (should be larger because of matching data at the wrong position/context ?
#gapPq=gapCost for gap inserts into target sequence (q-loci) (should be lower because of missing genes in context)?

SynBlast::MyUtils::checkStrEnd(\$bsourceDir, "/") && print STDERR ">blastresult directory name changed to " . $bsourceDir . "\n";

#########################################################################

my $dpColors = undef;

if(! ($bFl || $noRelScFl)) {
    @{$dpColors} = ();
    @{$dpColors->[0]} = ();
    @{$dpColors->[1]} = ();
    for(my $ci=12; $ci<129; $ci+=4) { 
	push(@{$dpColors->[0]}, [ (128-$ci, 127+$ci, 128-$ci) ]);   ### original green colors
	push(@{$dpColors->[1]}, [ (7+int((140-$ci)*(140-$ci) /66.), 7+int((140-$ci)*(140-$ci) /66.), 7+int((140-$ci)*(140-$ci) / 66.)) ]);   ### black and white and reversed meaning: dark means high scoring.4
    }
    push(@{$dpColors->[0]}, [ (100,200,244) ]);
    push(@{$dpColors->[1]}, [ (0,0,0) ]);
}

my @htmlpc=();
@htmlpc=split(/\//, $htmlFile) if(defined($htmlFile));
my $htmlpc="";
$htmlpc= ('../' x (scalar(@htmlpc)-1)) if(@htmlpc);
#print $htmlpc . "\n\n\n";


my @cssfile=();
my $cssfile=undef;
my $cssname="html.css";
if(defined($htmlFile)) {
  @cssfile=split(/\//, $htmlFile);
  $cssfile[-1]="";
  my $htmlDIR=join("/",@cssfile);
  if($htmlDIR && (! -d $htmlDIR)) {
	  mkdir $htmlDIR;
  }
  $cssfile[-1]= $cssname;
  $cssfile= join("/",@cssfile);
  
  if(! (-e $cssfile) ) {
    ### html.css not existing -> create default one...
    open(CSSF, '>', "" . $cssfile)  or die "Error while creating \"" . $cssfile . "\": $!\n";
    print CSSF <<"EOF";
%%%%% default style sheet for doSyntenyAlignment resulting html files %%%%%%%
body{
	font-family:"ms trebuchet",arial,sans-serif;
	font-size:90%;
	color:rgb(0,0,0);
	padding:0;
	margin:2em;
}

table{	border:2px solid #333; border-collapse:collapse; margin:.2em; }
table th{ border:1px solid #333; padding:.2em; font-weigth:bold;}
table td{ border:1px solid #333; padding:.2em;}

.overview{background:#f8f8f8;}
table.overview{ border:2px solid #333; border-collapse:collapse; margin:.2em; text-align: left; }

.params{ font-style:italic; background:#f8f8f8; }

.navigation{background:#f8f8f8; padding:0; margin:2px; font-family:sans-serif; font-size:small;}
table.navigation{ text-align:center; white-space:nowrap; border-style:none; padding:0px; margin:0px; background:#eee;}
table[class=navigation] td{border:0px solid #333; padding-left:0.2em; padding-rigth:0.2em; padding-bottom:0; padding-top:0;}

.aliresult{background:#f8f8f8;}
table.aliresult{ border:2px solid #333; border-collapse:collapse; margin:.2em; }
table[class=aliresult] th.refloci{ background-color:rgb(255,255,102);vertical-align:center; text-align:center;}
table[class=aliresult] td.refloci{ background-color:rgb(255,255,102);vertical-align:center; text-align:center;}
table[class=aliresult] th.targetloci{ background-color:rgb(51,255,51);vertical-align:center; text-align:center;}
table[class=aliresult] td.targetloci{ background-color:rgb(51,255,51);vertical-align:center; text-align:center;}
table[class=aliresult] td.notOptLoci{ background-color:rgb(128,80,80);vertical-align:center; }

table.aligraphic{ border:1px solid #333; padding:.2em; text-align:left; }
table[class=aligraphic] td{border:0px; margin:0; padding:0.3em; padding-bottom:0;}

.code{  font-family:monospace; margin:0em; padding:0em; font-size:x-small; color:rgb(0,0,0); }

#content{ color:#333; background:#f8f8f8; padding:1em; }

#footer{ font-family:monospace; border-top:#333 solid 1px; background:#eee; color:#333; padding:.5em; }

h1{ color:rgb(0,0,0); font-size:150%; font-family:georgia,sans-serif; }
h2{ font-family:georgia,sans-serif; color:rgb(0,0,0); font-size:120%; }
a:link{	color:#036; }
a:visited{ color:#369; }
a:active{ color:#333; }
a:hover{ background:#ffc; color:#000; }

span, pre{ background:#eee; padding:0em; }
EOF
    close CSSF;
  }
}


# creating list of queries (here named "exons") out of infoFile...
my @exons=();
my %queryIdx=(); # stores the position of query-key in exon-list
my @refExons=();
my @exonList=();


my($icolIdx
	, $infoH
	, $realFlankS
	, $realSlength) = SynBlast::MySyntenyUtils::infofile2Exonlist("${infoFile}"
																											, \@exons
																											, \%queryIdx
																											, \@refExons
																											, \@exonList
																											, $verboseFl
																											, "protID"
																											);



my @infoData = ( \@exons, \%queryIdx, \@refExons, \@exonList, $infoH, $icolIdx, $realFlankS, $realSlength);
my $refOrgName=$infoH->{"organism"};
my $refChrName=$infoH->{"seqregion"};


warn "-> WARNING: The reference data seems to be of a different Ensembl release version than the current API version (" .   $infoH->{'ensemblRelease'}   . " vs ${relStringShouldBe})! Be sure that you want that!\n" 
	unless((! $comparaFl) || (! exists($infoH->{'ensemblRelease'})) || ($relStringShouldBe eq $infoH->{'ensemblRelease'}));


# getting combinedPloci, i.e. data structure containing the (order of) queries grouped by meanPos (-mean flag) or start-end-positions (interv), to be used for the alignments to different putative syntenyRegions (simply filtered blastresults) later on
print STDERR ">creating p-loci data structure...";
my ($combPloci
    , $refPloci
    , $refProtStr
    , $refRLociIdxs, $maxPlociDist) = SynBlast::MySyntenyAln::getCombPlociRef(\@infoData, $verboseFl, $sFl);
print STDERR "done\n";
if(defined($maxDistStr)) {
  if(int($maxDistStr)) { ## set maximal Distance of columns not to separate orange in dotplot via constant from cmdline
    $maxDist= int($maxDistStr);
  } else {
    ## set to maximal distance of neighboring genes in reference multiplied with sizeFactor...
    $maxDist= $maxPlociDist * $locSizeFac if($maxPlociDist);
  }
}
  
  

#################################################################

my %resultHash=();
my $plociTable=undef; # contains later the overview on reference table result
my %comparaData=();

### if comparaFlag enabled, pre-store all member-structures for reference geneIDs
my @memberList=(); # contains only the RefGID if comparaAllFl not set
my $member_adaptor;
my $homologyAdapt;
if($comparaFl) {
    SynBlast::EnsEMBLaccess::loadRegistry($initF, $ehost, $euser);
      $member_adaptor = SynBlast::EnsEMBLaccess::getDBadaptor("Multi", "compara", "Member");
      $homologyAdapt = SynBlast::EnsEMBLaccess::getDBadaptor('Multi', 'compara', 'Homology');
      
      my @GIDsForCompara = @refExons;
      
      if($comparaAllFl) { 
	  @GIDsForCompara = @exonList;
      }
    			@GIDsForCompara = map($exons[$queryIdx{$_}]->[$icolIdx->{'geneID'}], @GIDsForCompara);  # get GeneIDs out of QueryIDs(ProteinIDs)
      foreach my $cGID (@GIDsForCompara) {
	  my $cmember=$member_adaptor->fetch_by_source_stable_id('ENSEMBLGENE', $cGID);
	  die "ERROR: could not get member for ${cGID}!\n" unless (defined $cmember);
	  #### die "ERROR:
	  push(@memberList, $cmember);
      }
  }

my ($curPNGdir, $cPNGsubdir);
my ($blHitDir, $blHitSubdir) = ("", "");
if(defined($htmlFile)) {
  $curPNGdir = $htmlFile . ".dir";
    my @cPNGsubdir = split(/\//,$curPNGdir);  
    $cPNGsubdir = $cPNGsubdir[-1];  # relative to html-file location
    $blHitDir = $curPNGdir . "/chainedHits";
    $blHitSubdir = $cPNGsubdir . "/chainedHits";
   
}


my ($trackDir,$tracksubDir);
if($trackerLimit && defined($trackFile)) {
  $trackDir= $trackFile . ".dir";
  my @cPNGsubdir = split(/\//,$trackDir);
  $tracksubDir= $cPNGsubdir[-1];  # relative to tracker-file location
  mkdir($trackDir) unless(-d $trackDir);
}

my %globalRankHits=(); ## saves (organism->queryID->[list of ranked globalHits])
my %globalRanks=(); ### saves (organism->queryID->chr/contig->[list of globalHits=[(rank number, startPos, endPos)]]


my %refscores=();
if(! $noRelScFl) {   ## by default, the best and second best within-reference hit/score is stored, to use it for intra/inter-score calculation and as selfScore (for relativeScore calculation)
	foreach my $protQID (@exonList) {
		my $QgeneID = $exons[$queryIdx{$protQID}]->[$icolIdx->{'geneID'}]; # get the corresponding geneID for current protQID
		if(! defined($refscores{$protQID})) { ### now get reference's two best blast loci for refscore hash
			my $blastFile;
			if(defined($ablastf)){
				$blastFile= "${bsourceDir}/" . $ablastf;
				$blastFile.= ".${refOrgName}.blastresult" if(substr($ablastf, -12) ne ".blastresult");
			} else {
				$blastFile = "${bsourceDir}/${protQID}*${refOrgName}.blastresult";
			}
			my @blastFiles= glob($blastFile);
			die "Error: ${blastFile} is not unique or could not be found!\n" 
					. ("#" x 74) . "\n# Please ensure to have within-reference blast hit data available,       #\n"
					. "# or use the \"-noRelScore\"-Flag for use of absolute blast scores values! #\n" . ("#" x 74) . "\n" unless (@blastFiles == 1);
			my $refglobal2Hits = SynBlast::MySyntenyAln::getBestBlastLoci(\@infoData
																			, $blastFiles[0]
																			, $protQID
																			, 0 # $scutoff   ### do not use scoreCutoff for assembled hits here, not to miss the second best hit
																			, 40 ###	$protOvl
																			, 0  # verboseFl=0 -> no output on screen here
																			, undef #restrChr
																			, 2 # 2 best global hits are saved
																			, 0 # noCompara here
																			, $refOrgName
																			, $QgeneID
																			, "-"  ## gene name 
																			, $rlocSizeFac  ### 1.1 	###	$locSizeFac #2 # rangeLimit factor
																			, $deepFl # 0 # no deep recursion version
																			, undef # no sliceadapt
																			, undef # no comparaDat
																			, undef    ### no restriction here ### 100000   ###  $globLocSize
																			, $wublastFl
																			);
			if($refglobal2Hits && (@{$refglobal2Hits} > 1)) {
				$refscores{$protQID}=  [ ($refglobal2Hits->[0]->{'score'}, $refglobal2Hits->[1]->{'score'}) ] ;
			} elsif($refglobal2Hits && @{$refglobal2Hits}) {
				$refscores{$protQID}=  [ ( $refglobal2Hits->[0]->{'score'}, undef) ];
			} else {
				die "ERROR: Could not determine any globally-best hit for query protein \"${protQID}\" "
					. "within reference blast hit data (${refOrgName})!\nPlease either\n"
					#. " -modify your cutoff/overlap parameters\n"
					. " -check for correctness/availability of reference blast hit data\n"
					. " -uncomment the protein entry in the annotation file (maybe a too short protein to be detectable by tblastn?)\n";
					
				$refscores{$protQID}=  [ ( undef, undef) ];
			}
			
			### now check for overlap of annotated reference positions and best-blast-hit position for current protQID
			## will issue a warning if selfscore is not "valid"
			if($refChrName eq $refglobal2Hits->[0]->{'chr'}) {
				
				my $QstartPos = $exons[$queryIdx{$protQID}]->[$icolIdx->{'startPos'}]; # get the corresponding startPos for current protQID
				my $QendPos = $exons[$queryIdx{$protQID}]->[$icolIdx->{'endPos'}]; # get the corresponding endPos for current protQID
				if(SynBlast::MyUtils::getOverlapSize($refglobal2Hits->[0]->{'startPos'},  $refglobal2Hits->[0]->{'endPos'},  $QstartPos, $QendPos) > 0) {
					 print STDERR " -> selfscore (${protQID}) successfully validated...\n";
				} else {
						warn "WARNING: Best-reference-hit does not overlap with coordinates from annotation file!\n"
							. " -> If the annotation file containes real chromosome coordinates for the entries, the selfscore for query protein \"${protQID}\" is NOT valid!\n";
				}
			} else {
				warn "WARNING: Best-reference-hit is not on the same chromosome/contig as the reference!\n"
							. " -> the selfscore for query protein \"${protQID}\" is NOT valid!\n";
			}
			##
			
		}
	}
}

#########################################################################################
for(my $ai=0; $ai<@ARGV; ++$ai) { # process all putative-synteny-inputfiles
    my $blastFile=$ARGV[$ai];
    
    ### get Info about synteny region
    my($ensemblStr, $synOrg, $synChr, $synStart, $synEnd) = SynBlast::MySyntenyUtils::getSynRegInfo($blastFile);

warn "-> WARNING: The target block data of ${blastFile} seems to be of a different Ensembl release version than the current API version (" .   $ensemblStr   . " vs ${relStringShouldBe})! Be sure that you want that!\n" 
	unless((! $comparaFl) || (! $ensemblStr) || ($relStringShouldBe eq $ensemblStr));


	$globalRanks{$synOrg}=undef if(! $globalRankFl);
	if( $globalRankFl && (! exists($globalRankHits{$synOrg}))) {
		### save global rank hits for new organism...
			#...
			print STDERR ">fetching global-besthit data for all queries against ${synOrg}...";
			foreach my $protQID (@exonList) {
				 my $QgeneID = $exons[$queryIdx{$protQID}]->[$icolIdx->{'geneID'}]; # get the corresponding geneID for current protQID
				 my $blastFile;
				if(defined($ablastf)){
							$blastFile= "${bsourceDir}/" . $ablastf;
							$blastFile.= ".${synOrg}.blastresult" if(substr($ablastf, -12) ne ".blastresult");
 		   		} else {
						$blastFile = "${bsourceDir}/${protQID}*${synOrg}.blastresult";
   				}
				my @blastFiles= glob($blastFile);
				die "Error: ${blastFile} is not unique or could not be found!\n" unless (@blastFiles == 1);
				$globalRankHits{$synOrg}->{$protQID} = SynBlast::MySyntenyAln::getBestBlastLoci(\@infoData
																			, $blastFiles[0]
																			, $protQID
																			, $scutoff
																			, $protOvl
																			, 0  # verboseFl=0 -> no output on screen here
																			, undef #restrChr
																			, $nglobal #10 # 10 best global hits are saved
																			, 0 # noCompara here
																			, $synOrg
																			, $QgeneID
																			, "-"  ## gene name 
																			, $locSizeFac #2 # rangeLimit factor
																			, $deepFl # 0 # no deep recursion version
																			, undef # no sliceadapt
																			, undef # no comparaDat
																			, $globLocSize
																			, $wublastFl
																			);
			}
			
			### prepare for comparison
			foreach my $gqID (keys %{$globalRankHits{$synOrg}}) {
				if(defined($globalRankHits{$synOrg}->{$gqID})) {
				 	for(my $ghR=0; $ghR< @{$globalRankHits{$synOrg}->{$gqID}}; ++$ghR) {
						my $cglobHit=$globalRankHits{$synOrg}->{$gqID}->[$ghR];
						push(@{$globalRanks{$synOrg}->{$gqID}->{$cglobHit->{'chr'}}}, [ ($cglobHit->{'rank'}, $cglobHit->{'startPos'}, $cglobHit->{'endPos'}, $cglobHit->{'ori'}, $cglobHit->{'score'}) ]); 
					}
				}
			}
			print STDERR "done\n";
	 } 
	


	#DataBrowser::browse(\%refscores);



    if($comparaFl) {
	### query compara db if new organism
	if(! exists($comparaData{$synOrg})) {
	    foreach my $cMembr (@memberList) {
		SynBlast::EnsEMBLaccess::storeHomologsForGID($cMembr, $cMembr->stable_id(), $synOrg, \%comparaData, $homologyAdapt);
	      }
	}
    }
    

    
    my ($synAlnScore
	, $refData
	, $HTMLdata
	, $alnData
	, $PNGdata
	, $DPpngRef
	, $pHTMLdata
	, $selfCompData
	, $assignedBlHits, $exonFdata, $assignedRange, $DPpngRefp, $sumRatio, $fsumRatio, $logSumRatio, $flogSumRatio)=SynBlast::MySyntenyAln::getSyntenyAln($combPloci
										   , $refPloci
										   , $refProtStr
										   , $refRLociIdxs
										   , \@infoData
										   , $blastFile
										   , $scutoff, $protOvl, $locSizeFac
										   , $verboseFl, $bFl, $sFl
										   , \@pfields, \@qfields
										   , $gapPp, $gapPq, $mismP, $pMisOri
										   , $ai +1
										   , \@ppfields
										   , $dpColors
										   , $printcFl
										   , $comparaAllFl
										   , $initF
										   , $comparaData{$synOrg}
										   , $blHitSubdir ### for link to blastHits in html-specific subdir...
										   , $globalRanks{$synOrg}
										   , (! $noRelScFl)
										   , $deepFl
										   , $maxDist
										   , $globLocSize
										   , \%refscores
						     );
    
    die "ERROR: undef AlnScore!!!\n" unless defined($synAlnScore);
    
    next unless($showallFl || $synAlnScore); ### ignore/skip 'empty' alignments/regions (where e.g. the score cutoff did not allow any matching target locus)
    
    
    $plociTable=$pHTMLdata if($pHTMLdata); #if($ai == 0);
	
    my @blastFile= split(/\//, $blastFile);
    
    ### save Results for current alignment in cEntry-Hash
    my %cEntry= ( 'synRegion' => $blastFile[-1],
		  'blastFile'  => $blastFile,
		  'alnScore' =>  $synAlnScore,
		  'synOrg' => $synOrg,
		  'synStart' => $synStart,
		  'synEnd' => $synEnd,
		  'synChr' => $synChr,
		  'ensemblSite' => ("http://" . ($ensemblStr ? ($ensemblStr . ".archive") : "www") . ".ensembl.org")
		  );
    $cEntry{'refData'} = $refData if(defined($refData));
    $cEntry{'HTMLtable'} = $HTMLdata if(defined($HTMLdata));
    $cEntry{'alnData'} = $alnData if(defined($alnData));
    $cEntry{'PNGref'} = $PNGdata if(defined($PNGdata));
    $cEntry{'DPpngref'} = $DPpngRef if(defined($DPpngRef));
    $cEntry{'DPpngrefp'} = $DPpngRefp if(defined($DPpngRefp));
    $cEntry{'selfComp'} = $selfCompData if(defined($selfCompData));
    $cEntry{'blHits'} = $assignedBlHits if(defined($assignedBlHits));
    $cEntry{'exonFdata'} = $exonFdata if(defined($exonFdata));
    $cEntry{'assignedRange'} = $assignedRange if(defined($assignedRange));
    $cEntry{'sumRatio'} = $sumRatio if(defined($sumRatio));
    $cEntry{'fsumRatio'} = $fsumRatio if(defined($fsumRatio));
    $cEntry{'logSumRatio'} = $logSumRatio if(defined($logSumRatio));
    $cEntry{'flogSumRatio'} = $flogSumRatio if(defined($flogSumRatio));

    $cEntry{'regID'} = $ai + 1;
  
#  push(@{$resultHash{$synAlnScore}}, \%cEntry);
  if($ratioFl) {
	  if($ratioFl == 1) {
		push(@{$resultHash{$sumRatio}}, \%cEntry);
	} elsif($ratioFl == 2) {
		push(@{$resultHash{$logSumRatio}}, \%cEntry);
	} elsif($ratioFl == 3) {
		push(@{$resultHash{$fsumRatio}}, \%cEntry);
	} else {
		$ratioFl = 0;
		#push(@{$resultHash{$flogSumRatio}}, \%cEntry);
		push(@{$resultHash{$synAlnScore}}, \%cEntry);
	}
  } else {
	  push(@{$resultHash{$synAlnScore}}, \%cEntry);
  }

###process next blastresult-file (note: infoFile is not changed within one program run)
}


#################################################################
my $diffOrgFl=0; # count different target organisms


my %comparaD=();
my %compOrgData=();
# get Compara-Results for all refData (i.e. the refProtein-Hits in aln overview table) at once (to avoid several connections)
# sort all refDats according to organism
foreach my $dkey (reverse (sort {$a <=> $b} (keys %resultHash))) {
    foreach my $cEntry (@{$resultHash{$dkey}}) {
	push(@{$compOrgData{$cEntry->{'synOrg'}}}, $cEntry->{'regID'});  # save regID to corresponding organism
	#store compara-relevant data in hash with key regID:
	$comparaD{$cEntry->{'regID'}} = $cEntry->{'refData'};
    }
}
$diffOrgFl = scalar(keys %compOrgData) -1;

if($comparaFl) {
    foreach my $compOrg (keys %compOrgData) {
#	#get connection to core-DB of current organism...
	my $slice_adaptor = SynBlast::EnsEMBLaccess::getDBadaptor($compOrg, "core", "Slice");
	if(defined($slice_adaptor)) {
         foreach my $cRegID (@{$compOrgData{$compOrg}}) {
	    # go through all regIDs and each entry for it for current org
	    foreach my $cRefDat (@{$comparaD{$cRegID}}) {
		my $coordType = ""; #"scaffold"; #empty string works for all other contig/scaffold/group types...
		if(int($cRefDat->{'chr'})) {
		    $coordType = "chromosome";
		}
		
		my ($txtResult, $resHash, $gID2Name) = SynBlast::EnsEMBLaccess::checkComparaData($comparaData{$compOrg}->{$cRefDat->{'refGeneID'}}, $slice_adaptor, $compOrg, $coordType, $cRefDat->{'chr'}, $cRefDat->{'startPos'}, $cRefDat->{'endPos'}, $cRefDat->{'refGeneID'}, 0, 2);
		
		
		chomp($txtResult);
			
		my $ensemblSite= "http://www.ensembl.org"; # should be the archive version of infoFile-version
		my $compResHtml = SynBlast::MyUtils::comparaRes2HTML($resHash, $compOrg, $ensemblSite, $gID2Name);
		$cRefDat->{'compara'} = [ ($txtResult, $compResHtml) ];
	    }
	}
      } else { print STDERR "Error: Could not connect to database for ${compOrg}!\n"; } 
    }
}




#################################################################
#my @addInfoColumns=(qw(chr startPos endPos hitsCount ori meanPos score protInfo compara));
# additional columns in overview table of alignments, regarding the aligned q-locus for the reference-loci
my @addInfoColumns=(qw(chr startPos endPos hitsCount queryCov meanIdentity ori score));# comparaResult));
#push(@addInfoColumns, "globalRank") if($globalRankFl);
push(@addInfoColumns, "compara") if($comparaFl);

#my ($curPNGdir, $cPNGsubdir);
if(defined($htmlFile)) {
    open(HTMLF, '>', "" . $htmlFile)  or die "Error while creating \"" . $htmlFile . "\": $!\n";
    
    print HTMLF <<"EOF";
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/html4/strict.dtd">
<html dir="ltr" lang="en">
<head><meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<title>${infoFile} Alignment Results</title>
<style type="text/css">
\@import './${cssname}';
</style>
</head>
<body>
<div id="content">
EOF

##### \@import './${htmlpc}html.css';


    print HTMLF "<div class=\"overview\"><a name=\"top\"></a>" 
	. "<div class=\"navigation\">" . ( shortLinks() ) . "</div>"
	. "<h4>Overview on processed synteny regions for ${infoFile}</h4>\n";
    print HTMLF "<table class=\"overview\">\n";
    
    my $regionsCounter=0;

# store anchors for html-navigation organism-wise
   my %regOrg=();

  my $cntListed=0; 



    print HTMLF "<tr><th>Rank</th><th>AlnOverview</th>";
	# . "<td>Synteny Region File</td>"
	#. "<th>AlnScore</th><th>TargetGenome</th>";
	#. "<th>AlnScore</th>";
	print HTMLF "<th>" . ( ($ratioFl == 0) ? "<u>AlnScore</u>" : "AlnScore" ) . "</th>";
	print HTMLF "<th>" . ( ($ratioFl == 1) ? "<u>RatioSum</u>" : "RatioSum" ) . "</th>" if($displRatio);
	print HTMLF "<th>" . ( ($ratioFl == 2) ? "<u>logRatioSum</u>" : "logRatioSum" ) . "</th>" if($displRatio);
	print HTMLF "<th>" . ( ($ratioFl == 3) ? "<u>focalRatioSum</u>" : "focalRatioSum" ) . "</th>" if($displRatio);
#	print HTMLF "<th>" . ( ($ratioFl == 4) ? "<u>focalLogRatioSum</u>" : "focalLogRatioSum" ) . "</th>" if($displRatio);
	print HTMLF "<th>TargetGenome</th>";
    if(@addInfoColumns) {
	print HTMLF "<th>" . join("</th><th>", @addInfoColumns) . "</th>";
    }
    print HTMLF "</tr>\n";
    
    ### print out the sorted alignment-overviews (descending in alnScore)...
    foreach my $dkey (reverse (sort {$a <=> $b} (keys %resultHash))) {
	foreach my $cEntry (@{$resultHash{$dkey}}) {

#####
	my @cOrgShort = split(/_/, $cEntry->{'synOrg'});
	my $cOrgShort = $cEntry->{'synOrg'};
	$cOrgShort = substr($cOrgShort[0], 0, 1) . substr($cOrgShort[1], 0, 3) if(@cOrgShort ==2);
### Mmu Macacca mulata or Mus musculus -> better Mmus and Mmul
	++$regOrg{$cOrgShort};
######
  ++${regionsCounter};


	    next if($htmlLimit && (${regionsCounter} > $htmlLimit)); ## only the best n alignments are listed in html file (-n option)
	    next if($htmlSpLimit && ($regOrg{$cOrgShort} > $htmlSpLimit)); ## only the best m alignments per species to be listed (-snumber option)
	     
	   		
			++$cntListed;	   		
	   		
	   		my $rowspStr="";
	   		if((@addInfoColumns) && exists($cEntry->{'refData'}) && (@{$cEntry->{'refData'}} > 1)) {
		    $rowspStr=" rowspan=\"" . int(@{$cEntry->{'refData'}}) . "\"";
		}
		my $curPNGfileD= ${curPNGdir} . "/region" . ${regionsCounter} . "_d.png";
		my $cPNGlinkD = "./${cPNGsubdir}/region" . ${regionsCounter} . "_d.png";
		
		my $curPNGfileS= ${curPNGdir} . "/region" . ${regionsCounter} . "_s.png";
		my $cPNGlinkS = "./${cPNGsubdir}/region" . ${regionsCounter} . "_s.png";
		
		my $curDPpngfile= ${curPNGdir} . "/region" . ${regionsCounter} . "_dp.png";
		my $curDPpngfilep= ${curPNGdir} . "/region" . ${regionsCounter} . "_dp_p.png";
		#my $cDPpnglink = "./${cPNGsubdir}/region" . ${regionsCounter} . "_dp.png";
	    
	    
		# prints map for overview png-file...
		print HTMLF ${$cEntry->{'PNGref'}->[0]->[1]};
		print HTMLF 
		    "<tr><td${rowspStr}><a href=\"#region_" . ${regionsCounter} . "\">" . ${regionsCounter} . "</a></td><td${rowspStr}>" 
		    . "<a href=\"#region_" . ${regionsCounter} . "\">"
		    . "<img alt=\"region${regionsCounter}, score=" . $cEntry->{'alnScore'} . "\" title=\"region${regionsCounter}, score=" . $cEntry->{'alnScore'} . "\" src=\"${cPNGlinkS}\" usemap=\"#regionID" . $cEntry->{'regID'} . "map0\">" 
		    . "</a>"
		    . "</td>";
		print HTMLF "<td${rowspStr}>" . $cEntry->{'alnScore'} . "</td>";
		
		print HTMLF "<td${rowspStr}>" .  (defined($cEntry->{'sumRatio'}) ? $cEntry->{'sumRatio'}  : 0 ) . "</td>"  if($displRatio);
		print HTMLF "<td${rowspStr}>" .  (defined($cEntry->{'logSumRatio'}) ? $cEntry->{'logSumRatio'}  : 0 ) . "</td>"  if($displRatio);
		print HTMLF "<td${rowspStr}>" .  (defined($cEntry->{'fsumRatio'}) ? $cEntry->{'fsumRatio'}  : 0 ) . "</td>"  if($displRatio);
		#print HTMLF "<td${rowspStr}>" .  (defined($cEntry->{'flogSumRatio'}) ? $cEntry->{'flogSumRatio'}  : 0 ) . "</td>"  if($displRatio);
		
		print HTMLF "<td${rowspStr}>" . $cEntry->{'synOrg'} . "</td>";
		###
		

		################################## write out png-data...#################################
		mkdir(${curPNGdir}) unless(-d  ${curPNGdir});
		if(defined($cEntry->{'blHits'})) {
		 mkdir($blHitDir) unless(-d $blHitDir);
		}
	

		open(PNGF, '>', "${curPNGfileS}") or die "Error while creating \"${curPNGfileS}\": $!\n";
		binmode PNGF;
		print PNGF ${$cEntry->{'PNGref'}->[0]->[0]} . "\n";
		close(PNGF);
		#
		open(PNGF, '>', "${curPNGfileD}") or die "Error while creating \"${curPNGfileD}\": $!\n";
		binmode PNGF;
		print PNGF ${$cEntry->{'PNGref'}->[1]->[0]} . "\n";
		close(PNGF);
		
		### write DotPlot File...###
		open(PNGF, '>', "${curDPpngfile}") or die "Error while creating \"${curDPpngfile}\": $!\n";
		binmode PNGF;
		print PNGF ${$cEntry->{'DPpngref'}->[0]} . "\n";
		close(PNGF);
		###
		### write printer-DotPlot File...###
		if($printcFl && defined($cEntry->{'DPpngrefp'}) ) {
			open(PNGF, '>', "${curDPpngfilep}") or die "Error while creating \"${curDPpngfilep}\": $!\n";
			binmode PNGF;
			print PNGF ${$cEntry->{'DPpngrefp'}->[0]} . "\n";
			close(PNGF);
		}
		###
		
		if(defined($cEntry->{'blHits'})) {
		  foreach my $cProtID (keys %{$cEntry->{'blHits'}}) {
		    my $cBlHitFile=${blHitDir} . "/rID" . $cEntry->{'regID'} . "_" 
			. $cProtID . "." . $cEntry->{'synOrg'} . ".chainedBlastHits.txt";
		    open(PNGF, '>', "${cBlHitFile}") or die "Error while creating \"${cBlHitFile}\": $!\n";
		    print PNGF "# chained/assembled and filtered blastHits for query ${cProtID} in synteny-alignment number ${regionsCounter} (against " . $cEntry->{'synOrg'} . ")\n";
		    foreach my $blHitL (@{$cEntry->{'blHits'}->{$cProtID}}) {
		      print PNGF join("\t", @{$blHitL}) . "\n";
		    }
		    close(PNGF);
		  }
		}


		
		
		
		
		if((@addInfoColumns) && exists($cEntry->{'refData'})) {
		    # contains additional refData, i.e. referenceQuery was aligned to some blasthit?
		    #print HTMLF "<td colspan=\"" . int(@addInfoColumns) . "\"><table><tbody>" if(@{$cEntry->{'refData'}} > 1);
		    my $joeZ=0;
		    #foreach my $cRefDat (@{$cEntry->{'refData'}}) {
		    ##now use pre-stored comparaResults...
		    foreach my $cRefDat (@{$comparaD{$cEntry->{'regID'}}}) {
			print HTMLF "<tr>" if(++${joeZ} > 1); #@{$cEntry->{'refData'}} > 1);
			
			foreach my $icol (@addInfoColumns) {
			    print HTMLF "<td>" 
				. ((defined($cRefDat->{$icol})) ? (($icol eq "compara") ? $cRefDat->{$icol}->[1] : $cRefDat->{$icol}) : "NoData") 
				. "</td>\n";
			}
			print HTMLF "</tr>" if(${joeZ} < int(@{$cEntry->{'refData'}}));  #@{$cEntry->{'refData'}} > 1);
		    }
		}
		else { # no hit data available -> print at least chromosome name of target region
		    foreach my $ic (@addInfoColumns) {
			print HTMLF "<td>" . (($ic eq 'chr') ?  $cEntry->{'synChr'} : "") . "</td>\n";
		    }
		}
		print HTMLF "</tr>\n";
	 
	}
 }
    
    print HTMLF "</table>\n";
    #print HTMLF "<span style=\"font-family: monospace;\">Another " . (${regionsCounter} - $htmlLimit) . " alignments not displayed here.</span>" if($htmlLimit && (${regionsCounter} - $htmlLimit > 0));
    print HTMLF "<span style=\"font-family: monospace;\">Another " 
            . (${regionsCounter} -  $cntListed) . " alignments not displayed here.</span>" if((${regionsCounter} -  $cntListed > 0));
    
    print HTMLF "</div>\n";
    



################################# PLOCI-TABLE ########################################

#### Overview on used reference gene set (p-loci, data out of infoFile)
    my $slDigits = length("${realSlength}");
    my $slDivC= SynBlast::MyUtils::max(0, $slDigits-2);
    my $slDiv = "1" . ("0" x ${slDivC});
    print HTMLF "<hr/>" 
	. "<div class=\"aliresult\">\n"
	. "<div class=\"navigation\"><a name=\"region_0\"></a>" . (shortLinks()) . "</div>" 
	. ${$plociTable} 
    . "total length of ref-loci (on slice) = ${realSlength} (" . (int(${realSlength}/$slDiv + 0.5)) . "e${slDivC})"
	. "</div>" 
	. "<hr/>\n";
    


print HTMLF <<"EOF";
<div class="params">
<h4>Parameter used for the loci-alignments</h4>
qLoci-Score_CutOff: ${scutoff} bits
<br/>qLoci-MaxProtOverlap: ${protOvl} aa
<br/>loci size factor: ${locSizeFac} (additional loci size constraint: ${globLocSize} nt)
<br/>loci-grouping type:
EOF
    
    print HTMLF (($sFl) ? "mean cds/blastHit position" : "loci intervalls") . "<br/>\n";
    print HTMLF "max distance between non-highlighted (orange) target columns: ${maxDist} nt<br/>\n";
    
  
  
if(! ($bFl || $noRelScFl)) {
  print HTMLF "RelativeScore color index:<br/><table style=\"font-size:small;text-align:center; vertical-align:center;\">";
  
	print HTMLF	"" . getColorIndexHTML($dpColors->[0], 1000);
	print HTMLF "" . getColorIndexHTML($dpColors->[1], 1000, 1) if($printcFl);
	print HTMLF "</table>\n";
}
  print HTMLF "</div>\n";
}
# end if(htmlfile...)
  


################################# TXT-output on STDOUT #######################################
### contains only the cluster/synRegion/Alignment overview data together with positions of focal gene of interest
my $regionsCounter=0;

# store anchors for html-navigation organism-wise
  my %regOrg=();
  
  print "#Overview on processed synteny regions for ${infoFile}\n"
    ,"# AlnScore\tQueryFile\tTargetGenome\tchr/contig";
print "\t" . join("\t", @addInfoColumns) if(@addInfoColumns);
print "\n";

if(defined($txtFile)) {
  open(TXTF, '>', "" . $txtFile)  or die "Error while creating \"" . $txtFile . "\": $!\n";
  print TXTF "# alignment results for ${infoFile}\n# Columns are as follows\n";
  print TXTF "#region\talnScore\tqueryFile\ttargetGenome\ttargetChr\tpNumber\tqNumber\tqueryID";
  print TXTF "\t" . join("\t", @pfields) if(@pfields);
  print TXTF "\t" . join("\t", @qfields) if(@qfields);
  print TXTF "\n##########################\n";
}


my $trackerInputStr="# tracker input file generated from ${infoFile} syntenyAlns\n\n";
SynBlast::EnsEMBLaccess::loadRegistry($initF, $ehost, $euser) if((! $comparaFl) && $trackerLimit && defined($trackFile));


foreach my $dkey (reverse (sort {$a <=> $b} (keys %resultHash))) {
    foreach my $cEntry (@{$resultHash{$dkey}}) {

	if(! ${regionsCounter}++ ) {
	  
	  if( ${bFl} && ( $cEntry->{'synOrg'} eq $refOrgName) && ($cEntry->{'synChr'} eq $refChrName)) {
		# best alignment region and refOrg and blastOrg are the same?
		## update protein.info file with the best SelfScore in column 10
		##...
		#
		if(exists($cEntry->{'selfComp'})) {
		    SynBlast::MySyntenyUtils::info2Infofile($infoFile, undef, $cEntry->{'selfComp'}, 1);
		  } else {
		      print STDERR "No selfcomparison data available (-b option)!\n";
		  }
	    } elsif($bFl) {
		##error
		die "ERROR (-b option): Best alignment's reference and target are not the same (either in organism or chromosome name)!\n";
	    }

	}

#####
	#	    ++$regOrg{$cEntry->{'synOrg'}};
	my @cOrgShort = split(/_/, $cEntry->{'synOrg'});
	my $cOrgShort = $cEntry->{'synOrg'};
	$cOrgShort = substr($cOrgShort[0], 0, 1) . substr($cOrgShort[1], 0, 3) if(@cOrgShort ==2);
### Mmu Macacca mulata or Mus musculus -> better Mmus and Mmul
	++$regOrg{$cOrgShort};
	
	
	################# STDOUT output (only for focal gene assignment)#######
	print "##############\n# " . $cEntry->{'synRegion'} . "\t" . $cEntry->{'alnScore'} . "\n";
	
	if(exists($cEntry->{'refData'})) {
	    foreach my $cRefDat (@{$comparaD{$cEntry->{'regID'}}}) {
		print $cEntry->{'alnScore'} . "\t" . ${infoFile} . "\t" . $cEntry->{'synOrg'} . "\t" . $cEntry->{'synChr'};
		foreach my $icol (@addInfoColumns) {
		    print "\t" . ((defined($cRefDat->{$icol})) ? (($icol eq "compara") ? $cRefDat->{$icol}->[0] : $cRefDat->{$icol}) : "NoData");
		}
		print "\n";
		print "#" . $cRefDat->{'OptHitInfo'} . "\n";
	    }
	} else {
	    print $cEntry->{'alnScore'} . "\t" . ${infoFile} . "\t" . $cEntry->{'synOrg'} . "\t" . $cEntry->{'synChr'} . "\n";
	}
	
################################### TXT-table of ALIGNMENT ######################################
	if(exists($cEntry->{'alnData'}) && defined($txtFile)) {
	  foreach my $plocc (@{$cEntry->{'alnData'}}) {
	    foreach my $plsub (@{$plocc}) {
	      print TXTF "${regionsCounter}\t" . $cEntry->{'alnScore'} . "\t" . ${infoFile}
	      . "\t" . $cEntry->{'synOrg'} . "\t" . $cEntry->{'synChr'};
	      print TXTF "\t" . join("\t", @{$plsub}) if(@{$plsub});
	      print TXTF "\n";
	    }
	  }
	  print TXTF "#\n##################################################################\n#\n";
	}
	
	
########
	if(defined($trackFile) && defined($cEntry->{'exonFdata'}) && ($regOrg{$cOrgShort} <= $trackerLimit)) {
	  my $cExonFile=${trackDir} . "/${cOrgShort}_" . $regOrg{$cOrgShort} . ".exon";
	  my $cExonFasta=${trackDir} . "/${cOrgShort}_" . $regOrg{$cOrgShort} . ".fasta";
	  my $cExonLink=${tracksubDir} . "/${cOrgShort}_" . $regOrg{$cOrgShort};
	  
	  my $ExonOrgN=$cEntry->{'synOrg'} . " " . $regOrg{$cOrgShort};
	  my $ExonOrgID="${cOrgShort}_" . $regOrg{$cOrgShort};
	  
	  ## retrieve genomic slice
	  my ($sliceStart, $sliceEnd, $assignedOri) = @{$cEntry->{'assignedRange'}};
	  if($trackOvl) {
	    ($sliceStart, $sliceEnd) = ($sliceStart -$trackOvl, $sliceEnd + $trackOvl);
	  }
	  my $sliceL= $sliceEnd - $sliceStart + 1;
	  my $slice_adaptor = SynBlast::EnsEMBLaccess::getDBadaptor($cEntry->{'synOrg'}, "core", "Slice");
	  if(defined($slice_adaptor)) {
	    my $coordsys="";
	    $coordsys = "chromosome" if(int($cEntry->{'synChr'}));
	    my $slice = $slice_adaptor->fetch_by_region($coordsys, $cEntry->{'synChr'}, $sliceStart, $sliceEnd);

	    ($sliceStart, $sliceEnd) = ($slice->start(), $slice->end());
	    $sliceL= $sliceEnd - $sliceStart + 1;

	    open(EXNF, '>', "${cExonFasta}") or die "Error while creating \"${cExonFasta}\": $!\n";
	    if($assignedOri) { ## 0 means same as reference cluster
	      print EXNF "> ${ExonOrgID} " . $cEntry->{'synOrg'} . " " . $cEntry->{'synChr'} . " ${sliceStart}-${sliceEnd} reverse complement\n";
	      print EXNF "" . $slice->invert->seq() . "\n";
	    }
	    else {
	      print EXNF "> ${ExonOrgID} " . $cEntry->{'synOrg'} . " " . $cEntry->{'synChr'} . " ${sliceStart}-${sliceEnd}\n";
	      print EXNF "" . $slice->seq() . "\n";
	    }
	    close(EXNF);
	  }
	  
	  ###
	  open(EXNF, '>', "${cExonFile}") or die "Error while creating \"${cExonFile}\": $!\n";
	  print EXNF "# exonfile for ${cOrgShort} (from ranked aln " . $regOrg{$cOrgShort} . ", total rank ${regionsCounter})\n";
	  foreach my $cEx (@{$cEntry->{'exonFdata'}}) {
	    print EXNF "<GEN>\n" . $cEx->[0] . "\n" . $cEx->[1] . "\n";
	    
	    if($assignedOri) { ## 0 means same as reference cluster
	      ### reverse complement coordinates have to be used here (different ori as reference cluster wrt slice/chromosome)
	      for(my $lii=@{$cEx} -1; $lii >= 2; --$lii) {
		print EXNF "" 
		    . ($sliceL - ($cEx->[$lii]->[1] - $sliceStart + 1) +1) 
		    . " "
		    . ($sliceL - ($cEx->[$lii]->[0] - $sliceStart + 1) +1) 
		    . "\n";
	      }
	    } else {
	      for(my $lii=2; $lii< @{$cEx}; ++$lii) {
		print EXNF "" . ($cEx->[$lii]->[0] - $sliceStart + 1) . " " . ($cEx->[$lii]->[1] - $sliceStart + 1) . "\n";
	      }
	    }
	  }
	  close(EXNF);
	  
	  
	  $trackerInputStr .= "<ORGANISM>\n";
	  $trackerInputStr .= "# NOTE:reverse strand wrt reference cluster!\n" if($assignedOri);
	  $trackerInputStr .= $ExonOrgN . "\n"
	      . $ExonOrgID . "\n"
	    #  . "2\n" ## which priority??
	      . $cExonLink . ".fasta\n"
	      . $cExonLink . ".exon\n";
	}
	
	

	
################################### HTML-table of ALIGNMENT ######################################
	


    	if(exists($cEntry->{'HTMLtable'}) && defined($htmlFile)) {
	  
	  next if($htmlLimit && (${regionsCounter} > $htmlLimit));
	  next if($htmlSpLimit && ($regOrg{$cOrgShort} > $htmlSpLimit));
	  
	  	 
	  my $cDPpnglink = "./${cPNGsubdir}/region" . ${regionsCounter} . "_dp.png";
	  my $cDPpnglinkp = "./${cPNGsubdir}/region" . ${regionsCounter} . "_dp_p.png";
	  my $cPNGlinkD = "./${cPNGsubdir}/region" . ${regionsCounter} . "_d.png";
	  print HTMLF "<div class=\"aliresult\">\n";
	  
	  print HTMLF ${$cEntry->{'DPpngref'}->[1]};

	if($printcFl && defined($cEntry->{'DPpngrefp'}) ) {
	    print HTMLF ${$cEntry->{'DPpngrefp'}->[1]};
	}
	  
	  # print map data for detailed overview picture
	  print HTMLF ${$cEntry->{'PNGref'}->[1]->[1]};
	  
	  print HTMLF 
	      "<hr/><div class=\"navigation\"><a name=\"region_" . ${regionsCounter} . "\"></a>";
	  print HTMLF "<a name=\"region_" . ($regOrg{$cOrgShort}) . "_${cOrgShort}\"></a>" if($diffOrgFl);
	  print HTMLF ( shortLinks($regionsCounter, $htmlLimit, $cOrgShort, $regOrg{$cOrgShort}, $diffOrgFl) ) 
	      . "</div>\n";
	  
	  print HTMLF "<b>region " . ${regionsCounter};
	  print HTMLF " (region " . $regOrg{$cOrgShort} . " for ${cOrgShort})" if($diffOrgFl);
	  
	  print HTMLF "</b><br/>\n"
	      . "<table class=\"aligraphic\"><tr>\n"
	      . "<td><img alt=\"region${regionsCounter}, score=" 
	      . $cEntry->{'alnScore'} . "\" title=\"region${regionsCounter}, score=" 
	      . $cEntry->{'alnScore'} . "\" src=\"${cDPpnglink}\" usemap=\"#regionID" . $cEntry->{'regID'} . "dpmap\"></td>\n";
	
	print HTMLF  "<td><img alt=\"region${regionsCounter}, score=" 
	      . $cEntry->{'alnScore'} . "\" title=\"region${regionsCounter}, score=" 
	      . $cEntry->{'alnScore'} . "\" src=\"${cDPpnglinkp}\" usemap=\"#regionID" . $cEntry->{'regID'} . "dpmapP\"></td>\n" if($printcFl);
	      
	print HTMLF "<td><a href=\"#region_" . ${regionsCounter} . "\">"
	      . "<img alt=\"region${regionsCounter}, score=" . $cEntry->{'alnScore'} . "\" title=\"region${regionsCounter}, score=" 
	      . $cEntry->{'alnScore'} . "\" src=\"${cPNGlinkD}\" usemap=\"#regionID" . $cEntry->{'regID'} . "map1\">" 
	      . "</a></td></tr></table>\n";
	  
	  print HTMLF 
	      "syntenyRegion: <b><a href=\"" 
	      . "./${htmlpc}" . $cEntry->{'blastFile'} . "\" target=\"blastF\">"
	      . $cEntry->{'synRegion'} . "</a></b>";
	  #	. "&nbsp;";
	  print HTMLF " [<a href=\""
	      . $cEntry->{'ensemblSite'} . "/" 
	      . $cEntry->{'synOrg'} . "/contigview?region=" . $cEntry->{'synChr'} . "&start=" 
	      . $cEntry->{'synStart'} . "&end=" . $cEntry->{'synEnd'} . "\" target=\"ensemblT\">"
	      . "<b>View in Ensembl</b></a>]";
	  
	  my $synLength = int(($cEntry->{'synEnd'} - $cEntry->{'synStart'} + 1) * 0.75);
	  my $synDigits = length("${synLength}");
	  my $synDivC= SynBlast::MyUtils::max(0, $synDigits-2);
	  my $synDiv = "1" . ("0" x ${synDivC});
	  my $synLstr = (int(${synLength}/$synDiv + 0.5)) . "e${synDivC}";
	  # print HTMLF "total length of target-synteny-region= ${synLength} (${synLstr})<br/>\n";
	  print HTMLF "<br/>retrieve target region: <span class=\"code\" style=\"font-size:small;\">" 
	      . "getEnsemblProteins.pl -h " 
	      .  $cEntry->{'synOrg'}
	  . " -l " . $cEntry->{'synChr'}
	  . " -i " . $cEntry->{'synStart'} . ":" . $cEntry->{'synEnd'}
	  . " -f " . $synLstr
	      . " some_geneID_to_be_selected"
	      . "</span><br/>\n";
	  print HTMLF  "scoreRatio_sum=<b>" .  $cEntry->{'sumRatio'} . "</b>\n" if($displRatio && defined($cEntry->{'sumRatio'}));
	  print HTMLF  "log_scoreRatio_sum=<b>" .  $cEntry->{'logSumRatio'} . "</b>\n" if($displRatio && defined($cEntry->{'logSumRatio'}));
	  print HTMLF  "focal_scoreRatio_sum=<b>" .  $cEntry->{'fsumRatio'} . "</b>\n" if($displRatio && defined($cEntry->{'fsumRatio'}));
	  #print HTMLF  "focal_log_scoreRatio_sum=<b>" .  $cEntry->{'flogSumRatio'} . "</b>\n" if($displRatio && defined($cEntry->{'flogSumRatio'}));
	  
	  print HTMLF ${$cEntry->{'HTMLtable'}} . "\n</div>\n";
	  
	  
	}
	
      }
  }



if(defined($htmlFile)) {
  print HTMLF <<"EOF";
<hr>
</div>
<div id="footer">
<div>html-output generated by ${main::PROGN}, ${main::VERSION}<br/>
on
EOF
print HTMLF (localtime(time)) . " \n";
print HTMLF "with command</div><div class=\"code\">" . $cmdlineWas . "</div>\n";
print HTMLF <<"EOF";
</div>
</body>
</html>
EOF
    
close(HTMLF);
}
close(TXTF) if(defined($txtFile));	

if($trackerLimit && defined($trackFile)) {
  open(TRAF, '>', "${trackFile}") or die "Error while creating \"${trackFile}\": $!\n";
  print TRAF $trackerInputStr . "\n";
  print TRAF "<GENS>\n" . join(",", @exonList) . "\n\n";
  close(TRAF);
}


exit 0;


sub shortLinks {
  my ($cReg, $lastReg, $cOrg, $cRegOrg, $diffOrgFl) = @_;
  $lastReg = 1000 if(! defined($lastReg));
  $diffOrgFl=0 if(! defined($diffOrgFl));
  my $htmltxt="<br/>";
  $htmltxt .= "<table class=\"navigation\"><tr><td>";
  $htmltxt .= "[<a href=\"#region_0\">ref-loci</a>]</td><td>[<a href=\"#top\">top</a>]</td><td>[<a href=\".\">up</a>]";
  $htmltxt .= "</td></tr>";
  
  if(defined($cReg)) {
    $htmltxt .= "<tr><td>";
    if($cReg > 1) {
      $htmltxt .= "[<a href=\"#region_" . ($cReg -1) . "\">previous</a>]</td>";
    } else { 
      $htmltxt .= "[previous]</td>";
    }
    $htmltxt .= "<td>$cReg</td>";
    #}  
    #if(defined($cReg)) {
    if((! $lastReg) or ($cReg < $lastReg)) {
      $htmltxt .= "<td>[<a href=\"#region_" . ($cReg +1) . "\">next</a>]</td>";
    } else {
      $htmltxt .= "<td>[next]</td>";
    }
    $htmltxt .= "</tr>";
  }
  #$htmltxt .= "<br/>";
  
  
  if($diffOrgFl && defined($cRegOrg)) {
    $htmltxt .= "<tr><td>";
    if($cRegOrg > 1) { 
      $htmltxt .= "[<a href=\"#region_" . ($cRegOrg -1) . "_${cOrg}\">previous ${cOrg}</a>]</td>";
    } else { 
      $htmltxt .= "[previous ${cOrg}]</td>";
    }
    $htmltxt .= "<td>${cRegOrg}</td>";
    if((! $lastReg) or ($cRegOrg < $lastReg)) {
      $htmltxt .= "<td>[<a href=\"#region_" . ($cRegOrg +1) . "_${cOrg}\">next ${cOrg}</a>]</td>";
    } else {
      $htmltxt .= "<td>[next ${cOrg}]</td>";
    }
    #$htmltxt .= "<br/>";
    $htmltxt .= "</tr>";	
  }
  $htmltxt .= "</table><br/>";
  return $htmltxt;
}

sub getColorIndexHTML {
	my($colArray, $maxV, $chTxtColFl)=@_;
	
	my $retstr="";
	my $mm=$maxV / (@{$colArray} -1);
	my $halfw=int(@{$colArray} / 2.);
	for(my $cid=0; $cid<@{ $colArray} -1; ++$cid) {
		$retstr .="<td style=\"background-color: rgb(" . join(", ", @{$colArray->[$cid]}) . ");";
		#$retstr .= "color: rgb(" . join(", ",  (255- $colArray->[$cid]->[0], 255- $colArray->[$cid]->[1],  255- $colArray->[$cid]->[2]))  . ");" if($chTxtColFl);
		$retstr .= "color:rgb(255,255,255);"  if($chTxtColFl &&  ($cid > $halfw));
		$retstr .="\"><=<br/>" . int(($cid + 1)*$mm) . "</td>";
	}
	 $retstr .= "<td style=\"background-color:rgb(" . join(", ", @{$colArray->[@{$colArray} -1]}) . ");\">relSc>" . int((@{$colArray}-1)*$mm) . "</td>";
	 $retstr ="<tr>" . $retstr . "</tr>";
	 return($retstr);
}

sub diff255 {
		my($rgbv)=@_;
		return(255-$rgbv);
	}
	
#################################################################################################

sub HELP_MESSAGE {
	pod2usage(-verbose => 1);
}
sub VERSION_MESSAGE {
	print $main::VERSION;
} 

#################################################################################################
