#!/usr/bin/env perl
###########################################################################
# createRegistry.conf.pl:
# useful to generate a Registry-configuration file out of a list of entries in csv-format (space separated entries):
# example format:
##
# #source file for createRegistry.conf.pl script creating the corresponding registry configuration file for use with ensembl-API
# #first data line is for column names
# species	dbname	group	aliases	host user port pass
# Aedes_aegypti	ensembl_aae	core	aedes_aegypty	maitai.bioinf.uni-leipzig.de	anonymous	3306 ensembl
###
# NOTE that for the more recent Ensembl versions (from rel48 on), the database port number on ensembldb.ensembl.org has changed from 3306 to 5306.

###########################################################################

print <<"EOT";
use strict;

use Bio::EnsEMBL::Utils::ConfigRegistry;
use Bio::EnsEMBL::DBSQL::DBAdaptor;
use Bio::EnsEMBL::Compara::DBSQL::DBAdaptor;


EOT


    
my $firstDefLine=-1;
my %colIdx=();
my @lastLine=();
my $cali=-1;
while(<>) {
    chomp;
    my @curF=split(/\s+/);
 


    if ($_ =~ m/^\#/) {
	@lastLine=@curF;
    } else {
	## first data line?
	if(! (++${firstDefLine})) {
	    # last line before first data line contains column description and order in table
	    # save col-idx in Hash
	    substr($lastLine[0], 0, 1) = "" if(substr($lastLine[0], 0, 1) eq "#");
	    shift(@lastLine) if($lastLine[0] eq "");
	    for(my $li=0; $li< @lastLine; ++$li) {
		$colIdx{$lastLine[$li]}=$li;
	    }
	}
	
	my %curF=('host' => "maitai.bioinf.uni-leipzig.de", 'user' => "joe", 'port' => 3306);
	my @fields = qw(species group dbname aliases port user host pass);
	
	foreach my $col (@fields) {
	    $curF{$col} = $curF[$colIdx{$col}] if(exists($colIdx{$col}) && ($colIdx{$col} < @curF));
	}
	my $aliases = undef;
	if(exists($curF{"aliases"})) {
	    @{$aliases} = split(/,/, $curF{"aliases"});
	    delete($curF{"aliases"});
	    }
	if(exists($curF{"species"})) {
	    $curF{"species"} = join(" ", split(/_/, $curF{"species"}));
	    
	    print "\n\n############# " . $curF{'species'} . "\n";
	    my $compFl=0;
	    if(index(lc($curF{'group'}), "compara") > -1) { 
		$compFl=1;
		delete($curF{'group'});
	    }
	    
	    print "new Bio::EnsEMBL" . (($compFl ? "::Compara" : "")) . "::DBSQL::DBAdaptor(\n";
	    my $lauf=0;
	    foreach my $col ( keys %curF) {
		print "-${col}    => " . (($col eq 'port') ? ($curF{$col}) : ("'" . $curF{$col} . "'"));
		print "" . ((++$lauf < (keys %curF)) ? "," : " );") . "\n";
	    }
	    if($aliases) {
		print "my " if(! ++$cali);
		print "\@aliases = ('" . join("', '", @{$aliases}) . "');\n";
		print "Bio::EnsEMBL::Utils::ConfigRegistry->add_alias(\n"
		    . "-species => \"". $curF{"species"} . "\",\n"
		    . "-alias => \\" . "@" . "aliases );\n\n";
	    }
	} else { die "ERROR!\n"; }
    }
}
print "1;\n";
