#!/usr/bin/env perl
use SynBlast;
our $PROGN=[split(/\//, $0)]->[-1];
$VERSION = "SynBlast " . $VERSION . "\n${PROGN}'s Time-stamp: <2007-02-25 11:36:11 joe>\nsee ${url} for updates.\n";
# 2006-03
# joe@bioinf.uni-leipzig.de
########################################################

=head1 NAME

getFreeHosts.pl - 

=head1 SYNOPSIS

B<getFreeHosts.pl> 
[B<-lI<MaxAllowedLoadPercent>>]
[B<-nI<NumberOfHostsAtOnce>>]
[B<--help>]
[B<--version>]
[hostprefix(es)]

=head1 REQUIRES

require Getopt::Std;
require Pod::Usage;

=head1 DESCRIPTION

...
needs ruptime utility installed

=head1 OPTIONS

=over 5

=item B<-l> I<Maximum_Allowed_Load_Value> [Integer]

default is 0 (in percent, i.e. load 1.00 is 100 percent)

=item B<HostnamePrefix(es)> [String(s)]

Only use hosts having certain prefixes

=head1 AUTHOR

Joerg Lehmann, <joe@bioinf.uni-leipzig.de>

=head1  SEE ALSO


=cut


use strict;
use Getopt::Std;
use SynBlast::MyUtils;
use Pod::Usage;


$Getopt::Std::STANDARD_HELP_VERSION = 1;
my %opts=();
getopts('l:n:', \%opts) || pod2usage(-verbose => 0);

#############################################################
our @excludehosts = qw();  ### <---- here you can preset hosts never to be used for blast searches 
#############################################################

my $hmload=1;
$hmload=$opts{"l"} if(exists($opts{"l"}));

my $maxN=10;
$maxN=$opts{"n"} if(exists($opts{"n"}));

our @hprefix=@ARGV;

my @hostdata;
my $retv=0;

if(! $retv) {
    @hostdata=split(/\n/, qx(ruptime -lr));
    sleep(1);
    $retv=testLoads(\@hostdata, $maxN);
}

exit(! $retv);


sub checkPrefixes {
    my($str2check, $preflref) = @_; 
    my $onePrefixFound=0;
    return(1) if( @{$preflref} == 0);
    foreach my $pref (@{$preflref}) {
	$onePrefixFound++ if(index($str2check, $pref) == 0);
    }
    return($onePrefixFound);
}


# returns success status
sub testLoads {
    my($hostr, $maxHN) = @_;
    my $li=0, my $hfound=0, my $lastP=1;
     while($lastP && ($li < @{$hostr}) && ($hfound < $maxHN)) {
	my @curHost=split(/\s+/, $hostr->[$li]);
	if(index(join(" ", @excludehosts), $curHost[0]) < 0) {
	    # host not in exclude list?
	    if(checkPrefixes($curHost[0], \@hprefix)) {
		# host starting with at least one of the prefixes, if defined?
		if(@curHost>3) { # host is up?
		    my @curLoad=split(/,/, $curHost[6]); 
		    if((int($curLoad[0] * 100)) <= SynBlast::MyUtils::max(10, $hmload) ) { #$hmload) {
			#print $hostdata[$li] . "\n";
			sleep(1);
			my @realUptime = split(/:/, qx(ssh $curHost[0] uptime)); 
			if(@realUptime > 2) { # no error with ssh 
			    my @realLoad=split(/,/, $realUptime[@realUptime -1]); 
			    if((int($realLoad[0] * 100)) <= $hmload) {
				print $curHost[0] . "\n";
				++$hfound;
			    }
			}
		    } else { $lastP=0; }
		}
	    }
	}
	$li++;
    }
    return($hfound);
}


#################################################################################################

sub HELP_MESSAGE {
	pod2usage(-verbose => 1);
}
sub VERSION_MESSAGE {
	print $main::VERSION;
} 
################################################################################
